/*****************************************************************************/

/*
 *
 *   Copyright (c) 2002, Smart Link Ltd.
 *   All rights reserved.
 *
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions
 *   are met:
 *
 *       1. Redistributions of source code must retain the above copyright
 *          notice, this list of conditions and the following disclaimer.
 *       2. Redistributions in binary form must reproduce the above
 *          copyright notice, this list of conditions and the following
 *          disclaimer in the documentation and/or other materials provided
 *          with the distribution.
 *       3. Neither the name of the Smart Link Ltd. nor the names of its
 *          contributors may be used to endorse or promote products derived
 *          from this software without specific prior written permission.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/*
 *
 *	amrmo_init.c  -  Smart Link Soft Modem amr,pci driver initialization.
 *
 *	Author: Sasha K (sashak@smlink.com)
 *
 */

/*****************************************************************************/

#include <linux/config.h>
#include <linux/module.h>
#include <linux/version.h>
#include <linux/init.h>
#include <linux/pci.h>
#include <linux/kernel.h>


#define FORCE_VENDOR_DEVICE 1
#undef CONFIG_PM

#include <modem.h>
#if MODEM_DEBUG
#include <modem_debug.h>
#undef printk
#define printk dprintf
#define AMRMO_DBG(fmt...) printk(KERN_DEBUG fmt)
#else
#define AMRMO_DBG(fmt...)
#endif

/* modem cards ids list */

/* Intel */
/* #define PCI_VENDOR_ID_INTEL                     0x8086 */
#define PCI_DEVICE_ID_ICHAA			0x2416
#define PCI_DEVICE_ID_ICHAB			0x2426
#define PCI_DEVICE_ID_BANISTER			0x7196
#define PCI_DEVICE_ID_ICH2			0x2446
#define PCI_DEVICE_ID_ICH3			0x2486
#define PCI_DEVICE_ID_ICH4			0x24C6

#define PCI_VENDOR_ID_STANDARD_MICROSYSTEM	0x1055	
#define PCI_DEVICE_ID_STANDARD_MICROSYSTEM	0x9178

/* AMD */
/* #define PCI_VENDOR_ID_AMD			0x1022 */
#define PCI_DEVICE_ID_AMD_ACLINK		0x7446

/* NVidia */
/* #define PCI_VENDOR_ID_NVIDIA			0x10DE */
#define PCI_DEVICE_ID_MCP			0x01C1

/* VIA */
/* #define PCI_VENDOR_ID_VIA			0x1106 */
#define PCI_DEVICE_ID_VIA			0x3068

/* SiS */
#define PCI_VENDOR_ID_SIS			0x1039
#define PCI_DEVICE_ID_SIS630			0x7013
#define PCI_DEVICE_ID_SIS960			0x7018

/* Avance */
#define PCI_VENDOR_ID_ALS300p			0x4005 // PCI_VENDOR_ID_AVANCE
#define PCI_DEVICE_ID_ALS300p			0x0308

/* ALi */
#define PCI_VENDOR_ID_ALI			0x10B9 // PCI_VENDOR_ID_AL
#define PCI_DEVICE_ID_ALI5450			0x5450
#define PCI_DEVICE_ID_ALI5451			0x5451
#define PCI_DEVICE_ID_ALI5457			0x5457
#define PCI_DEVICE_ID_ALI5459			0x5459
#define PCI_DEVICE_ID_ALI545A			0x545A

/* Philips */		
/* #define PCI_VENDOR_ID_PHILIPS			0x1131 */
#define PCI_DEVICE_ID_UCB1500			0x3400

/* RealTek */
/* #define PCI_VENDOR_ID_REALTEK			0x10EC */
#define PCI_DEVICE_ID_8101			0x8197

#define PCI_VENDOR_ID_SMARTLINK_1		0x163C
#define PCI_VENDOR_ID_SMARTLINK_2		0x10A5		
#define PCI_DEVICE_ID_3052			0x3052
enum {
	ALS300_CARD = 1,
	VIA3058_CARD,
	ALI5450_CARD,
	ALI5451_CARD,
	ALI5457_CARD,
	SIS630_CARD,
	SIS960_CARD,
	ICH_CARD,
	ICH4_CARD,
	RTL8197_CARD,
	NVIDIA_CARD,
	PHILIPS_CARD,
	ALI545A_CARD,
	SIL3052_CARD
};


struct amrmo_struct {
	unsigned id;
	const char *name;
	struct pci_dev *pci_dev;
	unsigned int irq;
	unsigned long iobase1;
	unsigned long iobase2;
	unsigned long membase;
	unsigned long memlen;
	void *card;
	struct modem *modem;
};


/* extern prototypes */
extern void amrmo_card_interrupt(void *card);
extern int amrmo_card_start(void *card);
extern int amrmo_card_stop(void *card);
extern int amrmo_card_ioctl(struct modem *m,unsigned cmd,unsigned long);
extern int amrmo_card_open(struct modem *m);
extern int amrmo_card_close(struct modem *m);
extern void *amrmo_card_create(int card_id,void *amrmo,struct modem *m);
extern void amrmo_card_delete(void *card);


/* internal data */
#ifdef FORCE_VENDOR_DEVICE
static unsigned short vendor = 0;
static unsigned short device = 0;
MODULE_PARM(vendor,"h");
MODULE_PARM_DESC(vendor, "Vendor ID");
MODULE_PARM(device,"h");
MODULE_PARM_DESC(device, "Device ID");
#endif

static const char *card_names[] = {
	0,
	"ALS300+", // ALS300_CARD
	"VIA3058",
	"ALI1535",
	"ALI1535",
	"ALI5457/9",
	"SiS630",
	"SiS960",
	"ICH",
	"ICH4",
	"RealTek8101",
	"NvidiaMCP",
	"Philips",
	"ALI545A",
	"SIL3052"
};


static struct pci_device_id amrmo_pci_tbl [] __devinitdata = {
	{PCI_VENDOR_ID_SMARTLINK_1, PCI_DEVICE_ID_3052,    /* 163c:3052 */
	 PCI_ANY_ID, PCI_ANY_ID, 0, 0, SIL3052_CARD},
	{PCI_VENDOR_ID_SMARTLINK_2, PCI_DEVICE_ID_3052,    /* 10a5:3052 */
	 PCI_ANY_ID, PCI_ANY_ID, 0, 0, SIL3052_CARD},
	{PCI_VENDOR_ID_PHILIPS,  PCI_DEVICE_ID_UCB1500,    /* 1131:3400 */
	 PCI_ANY_ID, PCI_ANY_ID, 0, 0, PHILIPS_CARD},
	{PCI_VENDOR_ID_REALTEK,  PCI_DEVICE_ID_8101,       /* 10ec:8197 */
	 PCI_ANY_ID, PCI_ANY_ID, 0, 0, RTL8197_CARD},
	{PCI_VENDOR_ID_ALI, PCI_DEVICE_ID_ALI5459,         /* 10b9:5459 */
	 PCI_ANY_ID, PCI_ANY_ID, 0, 0, ALI5457_CARD},
	{PCI_VENDOR_ID_SMARTLINK_1, PCI_DEVICE_ID_ALI5459, /* 163c:5459 */
	 PCI_ANY_ID, PCI_ANY_ID, 0, 0, ALI5457_CARD},
	{PCI_VENDOR_ID_SMARTLINK_2, PCI_DEVICE_ID_ALI5459, /* 10a5:5459 */
	 PCI_ANY_ID, PCI_ANY_ID, 0, 0, ALI5457_CARD},
	{PCI_VENDOR_ID_ALS300p, PCI_DEVICE_ID_ALS300p,     /* 4005:0308 */
	 PCI_ANY_ID, PCI_ANY_ID, 0, 0, ALS300_CARD},
	{PCI_VENDOR_ID_STANDARD_MICROSYSTEM,
		PCI_DEVICE_ID_STANDARD_MICROSYSTEM,        /* 1055:9178 */
	 PCI_ANY_ID, PCI_ANY_ID, 0, 0, ICH_CARD},
	{PCI_VENDOR_ID_AMD, PCI_DEVICE_ID_AMD_ACLINK,      /* 1022:7446 */
	 PCI_ANY_ID, PCI_ANY_ID, 0, 0, ICH_CARD},
	{PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_MCP,          /* 10de:01c1 */
	 PCI_ANY_ID, PCI_ANY_ID, 0, 0, NVIDIA_CARD},
	{PCI_VENDOR_ID_INTEL, PCI_DEVICE_ID_ICHAA,         /* 8086:2416 */
	 PCI_ANY_ID, PCI_ANY_ID, 0, 0, ICH_CARD},
	{PCI_VENDOR_ID_INTEL, PCI_DEVICE_ID_ICHAB,         /* 8086:2426 */
	 PCI_ANY_ID, PCI_ANY_ID, 0, 0, ICH_CARD},
	{PCI_VENDOR_ID_INTEL, PCI_DEVICE_ID_BANISTER,      /* 8086:7196 */
	 PCI_ANY_ID, PCI_ANY_ID, 0, 0, ICH_CARD},
	{PCI_VENDOR_ID_INTEL, PCI_DEVICE_ID_ICH2,          /* 8086:2446 */
	 PCI_ANY_ID, PCI_ANY_ID, 0, 0, ICH_CARD},
	{PCI_VENDOR_ID_INTEL, PCI_DEVICE_ID_ICH3,          /* 8086:2486 */
	 PCI_ANY_ID, PCI_ANY_ID, 0, 0, ICH_CARD},
       	{PCI_VENDOR_ID_INTEL, PCI_DEVICE_ID_ICH4,
	 PCI_ANY_ID, PCI_ANY_ID, 0, 0, ICH4_CARD},
	{PCI_VENDOR_ID_VIA, PCI_DEVICE_ID_VIA,             /* 1106:3068 */
	 PCI_ANY_ID, PCI_ANY_ID, 0, 0, VIA3058_CARD},
	{PCI_VENDOR_ID_SIS, PCI_DEVICE_ID_SIS630,          /* 1039:7013 */
	 PCI_ANY_ID, PCI_ANY_ID, PCI_CLASS_COMMUNICATION_MODEM<<8, 0xff00, SIS630_CARD},
	{PCI_VENDOR_ID_SIS, PCI_DEVICE_ID_SIS960,          /* 1039:7018 */
	 PCI_ANY_ID, PCI_ANY_ID, PCI_CLASS_COMMUNICATION_MODEM<<8, 0xff00, SIS960_CARD},
	//{PCI_VENDOR_ID_ALI, PCI_DEVICE_ID_ALI5450,
	// PCI_ANY_ID, PCI_ANY_ID, 0, 0, ALI5450_CARD},
	//{PCI_VENDOR_ID_ALI, PCI_DEVICE_ID_ALI5451,
	// PCI_ANY_ID, PCI_ANY_ID, 0, 0, ALI5451_CARD},
	{PCI_VENDOR_ID_ALI, PCI_DEVICE_ID_ALI5457,         /* 10b9:5457 */
	 PCI_ANY_ID, PCI_ANY_ID, 0, 0, ALI5457_CARD},
	{PCI_VENDOR_ID_ALI, PCI_DEVICE_ID_ALI545A,         /* 10b9:545a */
	 PCI_ANY_ID, PCI_ANY_ID, 0, 0, ALI545A_CARD},
	{0,}
};

MODULE_DEVICE_TABLE (pci, amrmo_pci_tbl);



/*
 *  Modem stuff
 *
 */


static int amrmo_modem_start (struct modem *m)
{
	struct amrmo_struct *amrmo = m->driver_data;
	return amrmo_card_start(amrmo->card);
}

static int amrmo_modem_stop (struct modem *m)
{
	struct amrmo_struct *amrmo = m->driver_data;
	return amrmo_card_stop(amrmo->card);
}

static int amrmo_modem_ioctl(struct modem *m, unsigned int cmd, unsigned long arg)
{
	struct amrmo_struct *amrmo = m->driver_data;
	return amrmo_card_ioctl(amrmo->card,cmd,arg);
}

static int amrmo_modem_open(struct modem *m)
{
	struct amrmo_struct *amrmo = m->driver_data;
	if (!amrmo->card)
		return -ENODEV;
	MOD_INC_USE_COUNT;
	amrmo_card_open(amrmo->card);
	return 0;
}


static int amrmo_modem_close(struct modem *m)
{
	struct amrmo_struct *amrmo = m->driver_data;
	amrmo_card_close(amrmo->card);
        MOD_DEC_USE_COUNT;	
	return 0;
}


static struct modem_driver amrmo_modem_driver = {
	name: "AMRMO modem",
	open:	amrmo_modem_open,
	close:	amrmo_modem_close,
	start:  amrmo_modem_start,
	stop:   amrmo_modem_stop,
	ioctl:	amrmo_modem_ioctl,
};


/*
 *  PCI stuff
 *
 */

static void amrmo_pci_interrupt(int irq, void *dev_id, struct pt_regs *regs)
{
        struct amrmo_struct *amrmo = (struct amrmo_struct *)dev_id;
	amrmo_card_interrupt(amrmo->card);
}

#ifdef CONFIG_PM
static int amrmo_pci_suspend(struct pci_dev *dev, u32 unused)
{
	AMRMO_DBG("amrmo_pci_suspend...\n");
	return 0;
}

static int amrmo_pci_resume(struct pci_dev *dev)
{
	AMRMO_DBG("amrmo_pci_resume...\n");
	return 0;
}
#endif /* CONFIG_PM */


static int __init amrmo_pci_probe(struct pci_dev *pci_dev, const struct pci_device_id *pci_id)
{
	struct amrmo_struct *amrmo;
	printk(KERN_INFO "amrmo: probe %04x:%04x %s : %s card...\n",
	       pci_id->vendor,pci_id->device,pci_dev->name,
	       card_names[pci_id->driver_data]);
#ifdef FORCE_VENDOR_DEVICE
	if ( (vendor && pci_id->vendor != vendor) ||
	     (device && pci_id->device != device) ) {
		printk(KERN_WARNING
		       "force configured for only %04x/%04x cards.\n",
		       vendor, device);
		return -ENODEV;
	}
#endif
	if ( pci_enable_device(pci_dev) ||
	     pci_dev->irq == 0 )
		return -ENODEV;

	amrmo = kmalloc(sizeof(*amrmo), GFP_KERNEL);
        if (!amrmo)
                return -ENOMEM;
        memset(amrmo, 0, sizeof(*amrmo));

	amrmo->id = pci_id->driver_data;
	amrmo->name = card_names[amrmo->id];
	amrmo->pci_dev = pci_dev;
	amrmo->irq     = pci_dev->irq;
	if (pci_resource_flags(pci_dev,0)&IORESOURCE_MEM) {
		amrmo->membase = pci_resource_start(pci_dev, 0);
		amrmo->iobase1 = pci_resource_start(pci_dev, 1);
	}
	else if (pci_resource_flags(pci_dev,1)&IORESOURCE_MEM) {
		amrmo->iobase1 = pci_resource_start(pci_dev, 0);
		amrmo->membase = pci_resource_start(pci_dev, 1);
	}
	else {
		amrmo->iobase1 = pci_resource_start(pci_dev, 0);
		amrmo->iobase2 = pci_resource_start(pci_dev, 1);
	}

	amrmo->modem = modem_create(&amrmo_modem_driver,pci_dev->name);
	if(!amrmo->modem) {
		kfree(amrmo);
		return -ENOMEM;
	}
	amrmo->modem->driver_data = amrmo;

	amrmo->card = amrmo_card_create(pci_id->driver_data,amrmo,amrmo->modem);
	AMRMO_DBG("amrmo_pci_probe: amrmo is %p. data %p, pci %p, io %lx, %lx.\n",
		  amrmo, amrmo->card, amrmo->pci_dev, amrmo->iobase1,amrmo->iobase2);
	if (!amrmo->card) {
		printk(KERN_ERR "amrmo: cannot init hw.\n");
		goto error_out;
	}

	pci_set_master(pci_dev);
        /* claim iospaces and irq */
	if(pci_request_regions(pci_dev,(char*)amrmo->name)) {
		printk(KERN_ERR "amrmo: failed request regions.\n");
		goto error_out;
	}
        if(request_irq(amrmo->irq, &amrmo_pci_interrupt,SA_SHIRQ,amrmo->name,amrmo)) {
		printk(KERN_ERR "amrmo: failed request_irq\n");
		pci_release_regions(pci_dev);
		goto error_out;
	}

	pci_set_drvdata(pci_dev, amrmo);
	return 0;

 error_out:
	amrmo_card_delete(amrmo->card);
	modem_delete(amrmo->modem);
	kfree(amrmo);
	pci_disable_device(pci_dev);
	return -ENODEV;
}

static void __exit amrmo_pci_remove(struct pci_dev *pci_dev)
{
	struct amrmo_struct *amrmo = pci_get_drvdata(pci_dev);
	void *card;
	AMRMO_DBG("amrmo: remove %p...\n", amrmo);
        free_irq(amrmo->irq, amrmo);
	pci_release_regions(pci_dev);
	pci_set_drvdata(pci_dev, NULL);
	pci_disable_device(pci_dev);
	card = amrmo->card;
	amrmo->card = 0;
	amrmo_card_delete(card);
	modem_delete(amrmo->modem);
	kfree(amrmo);
	return;
}

/*
 *  module stuff
 */

MODULE_AUTHOR("Smart Link Ltd.");
MODULE_DESCRIPTION("SmartLink HAMR5600,SmartPCI56/561 based modem driver");
MODULE_LICENSE("Smart Link Ltd.");


#define AMRMO_MODULE_NAME "slamrmo"

static struct pci_driver amrmo_pci_driver = {
	name:		AMRMO_MODULE_NAME,
	id_table:	amrmo_pci_tbl,
	probe:		amrmo_pci_probe,
	remove:		amrmo_pci_remove,
#ifdef CONFIG_PM
	suspend:	amrmo_pci_suspend,
	resume:		amrmo_pci_resume
#endif /* CONFIG_PM */
};

static int __init amrmo_init(void)
{
	struct pci_dev *dev;
	if (!pci_present())
		return -ENODEV;

	printk(KERN_INFO "SmartLink AMRMO modem.\n");

	/* fix me: how to prevent modem cards grabing by
	   serial driver? */
	pci_for_each_dev(dev) {
		if(pci_match_device(amrmo_pci_tbl, dev) &&
		   pci_dev_driver(dev)) {
			AMRMO_DBG("device %04x:%04x %s is used by %s: remove\n",
				  dev->vendor,dev->device, dev->name,
				  dev->driver?dev->driver->name:"");
			if (dev->driver && dev->driver->remove)
				dev->driver->remove(dev);
			dev->driver = NULL;
		}
	}

	if (!pci_register_driver(&amrmo_pci_driver)) {
		pci_unregister_driver(&amrmo_pci_driver);
                return -ENODEV;
	}
	return 0;
}

static void __exit amrmo_exit(void)
{
	AMRMO_DBG("amrmo: exit...\n");
	pci_unregister_driver(&amrmo_pci_driver);
}

module_init(amrmo_init);
module_exit(amrmo_exit);

EXPORT_NO_SYMBOLS;


