/*****************************************************************************/

/*
 *
 *   Copyright (c) 2002, Smart Link Ltd.
 *   All rights reserved.
 *
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions
 *   are met:
 *
 *       1. Redistributions of source code must retain the above copyright
 *          notice, this list of conditions and the following disclaimer.
 *       2. Redistributions in binary form must reproduce the above
 *          copyright notice, this list of conditions and the following
 *          disclaimer in the documentation and/or other materials provided
 *          with the distribution.
 *       3. Neither the name of the Smart Link Ltd. nor the names of its
 *          contributors may be used to endorse or promote products derived
 *          from this software without specific prior written permission.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/*
 *
 *	sysdep_common.c  --  sysdep API.
 *
 *	Author: Sasha K (sashak@smlink.com)
 *
 *
 */

/*****************************************************************************/

#include <linux/version.h>
#include <linux/module.h>

#include <linux/kernel.h>
#include <linux/string.h>
#include <linux/slab.h>
#include <linux/interrupt.h>

#include <linux_compat.h>

/* memory allocations */
void *sysdep_malloc (unsigned int size)
{
        void *mem;
        mem = kmalloc(size, in_interrupt() ? GFP_ATOMIC : GFP_KERNEL);
        if(!mem)
                printk(KERN_ERR "slmdm: cannot alloc %db mem\n", size);
        return mem;
}

void sysdep_free (void *mem)
{
	kfree(mem);
}

/* strings */
size_t sysdep_strlen(const char *s)
{
	return strlen(s);
}
char *sysdep_strcpy(char *d,const char *s)
{
	return strcpy(d,s);
}
char *sysdep_strcat(char *d, const char *s)
{
	return strcat(d,s);
}
int sysdep_strcmp(const char *s1,const char *s2)
{
	return strcmp(s1, s2);
}
char *sysdep_strstr(const char *s1,const char *s2)
{
	return strstr(s1,s2);
}
void *sysdep_memset(void *d,int c,size_t l)
{
	return memset(d,c,l);
}
void *sysdep_memcpy(void *d,const void *s,size_t l)
{
	return memcpy(d,s,l);
}
void *sysdep_memchr(const void *s,int c,size_t l)
{
	return memchr (s,c,l);
}

int sysdep_vsnprintf(char *str, unsigned size, const char *format, va_list ap)
{
        return vsnprintf(str, size, format, ap);
}

int sysdep_sprintf(char *buf, const char *fmt, ...)
{
	va_list args;
	int i;
	va_start(args, fmt);
	i=vsprintf(buf,fmt,args);
	va_end(args);
	return i;
}

/* export symbols */
EXPORT_SYMBOL_NOVERS(sysdep_malloc);
EXPORT_SYMBOL_NOVERS(sysdep_free);
EXPORT_SYMBOL_NOVERS(sysdep_strlen);
//EXPORT_SYMBOL_NOVERS(sysdep_strcpy);
EXPORT_SYMBOL_NOVERS(sysdep_strcmp);
//EXPORT_SYMBOL_NOVERS(sysdep_strcat);
EXPORT_SYMBOL_NOVERS(sysdep_memset);
EXPORT_SYMBOL_NOVERS(sysdep_memcpy);
EXPORT_SYMBOL_NOVERS(sysdep_sprintf);


/* fpu backup/restore */
/* FIXME: remove fpu operation and this code */

#include <asm/system.h>

/* fops backup/restore */
unsigned sysdep_fpu_save(char *state)
{
#ifdef __arch_um__
	unsigned ts = 0;
#else
	unsigned ts = read_cr0()&8;
        if(ts) {
		clts();
		__asm__ volatile("fninit ; fwait");
	}
	else {
		__asm__ volatile("fnsave %0 ; fninit ; fwait" : "=m" (*state));
	}
#endif
	return ts;
}

void sysdep_fpu_restore(unsigned ts, char *state)
{
#ifdef __arch_um__
#else
        if (ts) {
		__asm__ volatile("fwait");
		stts();
	}
        else {
		__asm__ volatile("frstor %0" : : "m" (*state));
	}
#endif
}

